﻿using System;
using System.Linq;
using Infragistics.Win.UltraWinTree;
using System.Drawing;
using Infragistics.Win;
using Fairmas.FairPayroll.Components.ManningGuide.Controller;
using Fairmas.GenericClient.Interfaces;
using Fairmas.Proxy.BusinessModel;

namespace Fairmas.FairPayroll.Components.ManningGuide
{
    /// <summary>
    /// This class provides formatting functionalities for the manning guide.
    /// </summary>
    internal static class ManningGuideTreeFormatter
    {
        #region Internal Members

        /// <summary>
        /// This method sets up the styles of a column set within the manning guide tree.
        /// </summary>
        /// <param name="columnSet">The column set to set the styles for.</param>
        /// <param name="application">The current application.</param>
        /// <param name="selectedYear">The current selected year in the manning guide.</param>
        internal static void SetupColumnSet(UltraTreeColumnSet columnSet, IApplication application, int selectedYear)
        {
            if (columnSet == null)
                throw new ArgumentNullException("columnSet");

            if (application == null)
                throw new ArgumentNullException("application");

            columnSet.Columns["ManningGuideLineType"].Visible = false;
            columnSet.ColumnSettings.AllowSorting = Infragistics.Win.DefaultableBoolean.False;
            columnSet.BorderStyleCell = Infragistics.Win.UIElementBorderStyle.Rounded4;
            columnSet.CellAppearance.BorderColor = Color.Transparent;
            columnSet.ActiveCellAppearance.BorderColor = Color.Black;

            if (columnSet.Key == "Departments")
                SetupDepartmentsColumnSet(columnSet, application, selectedYear);
            else if (columnSet.Key == Resources.Resources.CAPTION_InputTools)
                SetupInputToolsColumnSet(columnSet);
            else if (columnSet.Key == "Positions")
                SetupPositionsColumnSet(columnSet);
            else if (columnSet.Key == "PositionOfManningGuideDataSetRelation")
                SetupManningGuidesColumnSet(columnSet);
            else if (columnSet.Key == "ManningGuideOfSalaryComponentDataSetRelation")
                SetupSalaryComponentsColumnSet(columnSet);
            else if (columnSet.Key == "SalaryComponentOfEmployeeBenefitDataSetRelation")
                SetupEmployeeBenefitsColumnSet(columnSet);
        }

        /// <summary>
        /// This method sets up the styles of a tree node.
        /// </summary>
        /// <param name="treeNode">The tree node to set up.</param>
        /// <param name="manningGuideTabController">The controller of the manning guide.</param>
        internal static void SetupManningGuideDataNode(UltraTreeNode treeNode, ManningGuideTabController manningGuideTabController)
        {
            if (treeNode == null)
                throw new ArgumentNullException("treeNode");

            if (manningGuideTabController == null)
                throw new ArgumentNullException("manningGuideTabController");

            var isRelationNodeWithoutChilds = treeNode.Text.ToLower().Contains("DataSetRelation") && treeNode.Nodes.Count == 0;
            if (isRelationNodeWithoutChilds)
            {
                treeNode.Visible = false;
                return;
            }

            SetupManningGuideDataNodesForDepartmentAndTotals(treeNode);
            SetupManningGuideDataNodesForInputTools(treeNode);
            SetupManningGuideDataNodesForPositionData(treeNode, manningGuideTabController);
            SetupManningGuideDataNodesForGrossEmployeeBenefit(treeNode, manningGuideTabController);
            SetupManningGuideDataNodesForCounts(treeNode, manningGuideTabController);
            SetupManningGuideDataNodesForVacation(treeNode, manningGuideTabController);
            SetupManningGuideDataNodesForPersonalPay(treeNode, manningGuideTabController);
            SetupManningGuideDataNodesForHolidays(treeNode, manningGuideTabController);
            SetupManningGuideDataNodesForTotals(treeNode);
        }

        internal static void MarkEmployeeCellsWhereCommentAvailable(UltraTreeNode positionNode)
        {
            //var employeeNodes = positionNode.Nodes.All.Where(node => ((UltraTreeNode)node).Text == ManningGuideLineType.Employee.ToString()).ToList();

            //foreach (UltraTreeNode node in employeeNodes)
            //{
            //    foreach (UltraTreeNodeCell cell in node.Cells)
            //    {
            //        //cell.Appearance.ImageBackground = Resources.Resources.TestCornerManningGuideCell;
            //        cell.Appearance.ImageBackgroundStyle = Infragistics.Win.ImageBackgroundStyle.Centered;
            //        cell.Appearance.ImageBackgroundAlpha = Infragistics.Win.Alpha.Opaque;
            //    }
            //}
        }

        #endregion

        #region Private Members

        /// <summary>
        /// This method sets up the style of total tree nodes.
        /// </summary>
        /// <param name="treeNode">The tree node to set up.</param>
        private static void SetupManningGuideDataNodesForTotals(UltraTreeNode treeNode)
        {
            if (treeNode == null)
                throw new ArgumentNullException("treeNode");

            if (treeNode.Cells.Count == 0)
                return;

            var lineType = (ManningGuideLineType)Enum.Parse(typeof(ManningGuideLineType), treeNode.Cells["ManningGuideLineType"].Value.ToString());
            
            if (lineType == ManningGuideLineType.TotalSalaryAndWages || lineType == ManningGuideLineType.TotalBonus || lineType == ManningGuideLineType.TotalPTEB
                || lineType == ManningGuideLineType.TotalPayroll || lineType == ManningGuideLineType.TotalCount || lineType == ManningGuideLineType.TotalHoursVacation
                || lineType == ManningGuideLineType.TotalHoursSickness || lineType == ManningGuideLineType.TotalHours || lineType == ManningGuideLineType.TotalEFTE || lineType == ManningGuideLineType.TotalFTE
                || lineType == ManningGuideLineType.TotalVacationAccrual || lineType == ManningGuideLineType.TotalVacationTakenMonetary || lineType == ManningGuideLineType.TotalHolidayAccrual || lineType == ManningGuideLineType.TotalHolidayTakenMonetary
                || lineType == ManningGuideLineType.TotalHolidayTakenNominal || lineType == ManningGuideLineType.TotalVacationTakenNominal || lineType == ManningGuideLineType.TotalVacationEntitlement)
            {
                treeNode.Override.NodeAppearance.ForeColor = Constants.COLOR_ManningGuideTreeTotal;
                treeNode.Override.NodeAppearance.FontData.Bold = DefaultableBoolean.False;
                treeNode.Cells["Name"].Appearance.FontData.Bold = DefaultableBoolean.True;
            }

            if (lineType == ManningGuideLineType.ServiceCharge)
            {
                treeNode.Override.NodeAppearance.ForeColor = Constants.COLOR_ManningGuideTreeTotal;
                treeNode.Override.NodeAppearance.FontData.Bold = DefaultableBoolean.False;
            }
        }

        /// <summary>
        /// This method sets up the style of a tree node containing position data (positions, salaries etc.).
        /// </summary>
        /// <param name="treeNode">The tree node to set up.</param>
        /// <param name="manningGuideTabController">The controller of the manning guide.</param>
        private static void SetupManningGuideDataNodesForPositionData(UltraTreeNode treeNode, ManningGuideTabController manningGuideTabController)
        {
            if (treeNode == null)
                throw new ArgumentNullException("treeNode");

            if (manningGuideTabController == null)
                throw new ArgumentNullException("manningGuideTabController");

            if (treeNode.BandName == Resources.Resources.CAPTION_Positions)
            {
                var isRealPosition = treeNode.Cells.Count > 0 && !string.IsNullOrEmpty(treeNode.Cells[1].Text);
                if (isRealPosition)
                {
                    treeNode.Override.NodeAppearance.FontData.Bold = Infragistics.Win.DefaultableBoolean.True;
                    treeNode.Override.NodeAppearance.ForeColor = Constants.COLOR_ManningGuideTreePositionName;
                }

                treeNode.Override.NodeAppearance.BackColor = Constants.COLOR_ManningGuideTreePositionBackground; // Color.FromArgb(180, 210, 215);
                treeNode.Override.CellClickAction = CellClickAction.SelectNodeOnly;
            }
            else if (treeNode.BandName == "PositionOfManningGuideDataSetRelation")
            {
                treeNode.Override.NodeAppearance.BackColor = Constants.COLOR_ManningGuideEmployee;
                treeNode.Nodes.Override.Sort = SortType.Ascending;
            }
            else if (treeNode.BandName == "ManningGuideOfSalaryComponentDataSetRelation")
            {
                treeNode.Override.NodeSpacingBefore = 0;

                var idstring = treeNode.Cells["SalaryComponentId"].Value.ToString();

                if (idstring.Contains("."))
                    idstring = idstring.Split('.')[0];               

                int salaryComponentId;
                var salaryComponent = null as SalaryComponent;
                if (int.TryParse(idstring, out salaryComponentId))
                    salaryComponent= manningGuideTabController.GetSalaryComponent(salaryComponentId);                 

                if (salaryComponent != null && salaryComponent.BonusCalculation)
                {
                    treeNode.AllowCellEdit = AllowCellEdit.ActivateOnly;
                    treeNode.Override.NodeAppearance.BackColor = Constants.COLOR_ManningGuideViewLineBackground;
                    return;
                }

                treeNode.Cells["Name"].Appearance.BackColor = Constants.COLOR_ManningGuideViewLineBackground;

                SetMonthAvailability(treeNode, "MonthPlanningUnitId1", "Jan", manningGuideTabController);
                SetMonthAvailability(treeNode, "MonthPlanningUnitId2", "Feb", manningGuideTabController);
                SetMonthAvailability(treeNode, "MonthPlanningUnitId3", "Mar", manningGuideTabController);
                SetMonthAvailability(treeNode, "MonthPlanningUnitId4", "Apr", manningGuideTabController);
                SetMonthAvailability(treeNode, "MonthPlanningUnitId5", "May", manningGuideTabController);
                SetMonthAvailability(treeNode, "MonthPlanningUnitId6", "Jun", manningGuideTabController);
                SetMonthAvailability(treeNode, "MonthPlanningUnitId7", "Jul", manningGuideTabController);
                SetMonthAvailability(treeNode, "MonthPlanningUnitId8", "Aug", manningGuideTabController);
                SetMonthAvailability(treeNode, "MonthPlanningUnitId9", "Sep", manningGuideTabController);
                SetMonthAvailability(treeNode, "MonthPlanningUnitId10", "Oct", manningGuideTabController);
                SetMonthAvailability(treeNode, "MonthPlanningUnitId11", "Nov", manningGuideTabController);
                SetMonthAvailability(treeNode, "MonthPlanningUnitId12", "Dec", manningGuideTabController);

                var isPreviousSalaryComponent = treeNode.Cells.Count > 0 && treeNode.Cells["ManningGuideLineType"].Value.ToString() == ManningGuideLineType.PreviousNormalSalaryComponent.ToString();
                if (isPreviousSalaryComponent)
                {
                    var cell = treeNode.Cells["Name"];
                    cell.AllowEdit = AllowCellEdit.ActivateOnly;

                    treeNode.Override.NodeAppearance.BackColor = Constants.COLOR_ManningGuideViewLineBackground;
                    treeNode.Override.NodeAppearance.ForeColor = Constants.COLOR_ManningGuidePreviousViewLineForeground;
                }
            }
            else if (treeNode.BandName == "SalaryComponentOfEmployeeBenefitDataSetRelation")
            {
                treeNode.Override.NodeAppearance.BackColor = Constants.COLOR_ManningGuideViewLineBackground;
            }
        }

        /// <summary>
        /// This method sets up the style of a tree node containing input tools.
        /// </summary>
        /// <param name="treeNode">The tree node to set up.</param>
        private static void SetupManningGuideDataNodesForInputTools(UltraTreeNode treeNode)
        {
            if (treeNode == null)
                throw new ArgumentNullException("treeNode");

            if (treeNode.BandName == Resources.Resources.CAPTION_InputTools)
            {
                if (treeNode.Text == Resources.Resources.CAPTION_InputTools)
                {
                    if (treeNode.Nodes.Count == 0)
                        treeNode.Visible = false;

                    treeNode.Override.NodeSpacingBefore = 0;
                    treeNode.Override.NodeSpacingAfter = 0;
                    treeNode.Override.NodeAppearance.BackColor = Constants.COLOR_ManningGuideTreeEditToolLineBackground;
                    treeNode.Override.NodeAppearance.ForeColor = Constants.COLOR_ManningGuideTreeEditToolLine;
                    treeNode.Override.CellClickAction = CellClickAction.SelectNodeOnly;
                }
                else
                {
                    treeNode.Cells["Jan"].AllowEdit = AllowCellEdit.Full;
                    treeNode.Cells["Feb"].AllowEdit = AllowCellEdit.Full;
                    treeNode.Cells["Mar"].AllowEdit = AllowCellEdit.Full;
                    treeNode.Cells["Apr"].AllowEdit = AllowCellEdit.Full;
                    treeNode.Cells["May"].AllowEdit = AllowCellEdit.Full;
                    treeNode.Cells["Jun"].AllowEdit = AllowCellEdit.Full;
                    treeNode.Cells["Jul"].AllowEdit = AllowCellEdit.Full;
                    treeNode.Cells["Aug"].AllowEdit = AllowCellEdit.Full;
                    treeNode.Cells["Sep"].AllowEdit = AllowCellEdit.Full;
                    treeNode.Cells["Oct"].AllowEdit = AllowCellEdit.Full;
                    treeNode.Cells["Nov"].AllowEdit = AllowCellEdit.Full;
                    treeNode.Cells["Dec"].AllowEdit = AllowCellEdit.Full;

                    treeNode.Override.NodeSpacingBefore = 0;

                    if (treeNode.Text == Resources.Resources.CAPTION_SicknessInputTool)
                        treeNode.Override.NodeSpacingAfter = 8;
                    else
                        treeNode.Override.NodeSpacingAfter = 0;

                    treeNode.Override.NodeAppearance.BackColor = Constants.COLOR_ManningGuideTreeEditToolLineBackground;
                    treeNode.Override.NodeAppearance.ForeColor = Constants.COLOR_ManningGuideTreeEditToolLine;
                    treeNode.Override.CellClickAction = CellClickAction.EditCellSelectText;
                }
            }
        }

        /// <summary>
        /// This method sets up the style of a tree node containing the department or totals.
        /// </summary>
        /// <param name="treeNode">The tree node to set up.</param>
        private static void SetupManningGuideDataNodesForDepartmentAndTotals(UltraTreeNode treeNode)
        {
            if (treeNode == null)
                throw new ArgumentNullException("treeNode");

            //treeNode.Control.RunInUIThread(() =>
            //    {
                    if (treeNode.Text == "Total")
                    {
                        treeNode.Override.NodeAppearance.FontData.Bold = Infragistics.Win.DefaultableBoolean.True;
                        treeNode.Override.NodeSpacingBefore = 15;
                        treeNode.Override.NodeAppearance.BackColor = Constants.COLOR_ManningGuideTreeTotalBackground;
                        treeNode.Override.NodeAppearance.ForeColor = Constants.COLOR_ManningGuideTreeTotal;
                        treeNode.Override.CellClickAction = CellClickAction.SelectNodeOnly;
                    }

                    treeNode.Override.NodeSpacingAfter = 0;
                    treeNode.Expanded = true;

                    if (treeNode.BandName == "Departments")
                    {
                        treeNode.Override.CellClickAction = CellClickAction.SelectNodeOnly;
                        treeNode.Override.NodeAppearance.BackColor = Constants.COLOR_ManningGuideTreeDepartmentBackground;
                        treeNode.Override.NodeAppearance.ForeColor = Constants.COLOR_ManningGuideTreeDepartmentForeground;
                        treeNode.Override.NodeAppearance.FontData.Bold = Infragistics.Win.DefaultableBoolean.True;
                    }
                //});
        }

        /// <summary>
        /// This method sets up the style of a tree node containing the count or FTE count.
        /// </summary>
        /// <param name="treeNode">The tree node to set up.</param>
        /// <param name="manningGuideTabController">The controller of the manning guide tab.</param>
        private static void SetupManningGuideDataNodesForCounts(UltraTreeNode treeNode, ManningGuideTabController manningGuideTabController)
        {
            if (treeNode == null)
                throw new ArgumentNullException("treeNode");

            if (manningGuideTabController == null)
                throw new ArgumentNullException("manningGuideTabController");

            var isCountLine = treeNode.Cells.Count > 0 && treeNode.Cells["ManningGuideLineType"].Value.ToString() == ManningGuideLineType.Count.ToString();
            if (isCountLine)
            {
                var id = 0;
                int.TryParse(treeNode.Cells["ManningGuideId"].Value.ToString(), out id);
                var manningGuide = manningGuideTabController.AditionalData.ContainsKey(id) ? manningGuideTabController.AditionalData[id] : null as AdditionalManningGuideValue;
                var isAutoGenerated = manningGuide != null && manningGuide.IsAutoGenerated;

                if (isAutoGenerated)
                {
                    treeNode.Override.NodeAppearance.BackColor = Constants.COLOR_ManningGuideViewLineBackground;
                }
                else
                {
                    var cell = treeNode.Cells["Name"];
                    cell.AllowEdit = AllowCellEdit.ActivateOnly;
                    cell.Appearance.BackColor = Constants.COLOR_ManningGuideViewLineBackground;

                    foreach (var month in Constants.MONTH_AllForDisplaying)
                    {
                        cell = treeNode.Cells[month];
                        cell.AllowEdit = AllowCellEdit.Full;
                    }
                }
            }

            var isPreviousCountLine = treeNode.Cells.Count > 0 && treeNode.Cells["ManningGuideLineType"].Value.ToString() == ManningGuideLineType.PreviousCount.ToString();
            if (isPreviousCountLine)
            {
                var cell = treeNode.Cells["Name"];
                cell.AllowEdit = AllowCellEdit.ActivateOnly;

                treeNode.Override.NodeAppearance.BackColor = Constants.COLOR_ManningGuideViewLineBackground;
                treeNode.Override.NodeAppearance.ForeColor = Constants.COLOR_ManningGuidePreviousViewLineForeground;
            }

            var isFteCountLine = treeNode.Cells.Count > 0 && treeNode.Cells["ManningGuideLineType"].Value.ToString() == ManningGuideLineType.FteCount.ToString();
            if (isFteCountLine)
                treeNode.Override.NodeAppearance.BackColor = Constants.COLOR_ManningGuideViewLineBackground;
        }

        /// <summary>
        /// This method sets up the style of a tree node containing the gross employee benefit.
        /// </summary>
        /// <param name="treeNode">The tree node to set up.</param>
        /// <param name="manningGuideTabController">The controller of the manning guide tab.</param>
        private static void SetupManningGuideDataNodesForGrossEmployeeBenefit(UltraTreeNode treeNode, ManningGuideTabController manningGuideTabController)
        {
            if (treeNode == null)
                throw new ArgumentNullException("treeNode");

            if (manningGuideTabController == null)
                throw new ArgumentNullException("manningGuideTabController");

            var isGrossEmployeeBenefitLine = treeNode.Cells.Count > 0 && treeNode.Cells["ManningGuideLineType"].Value.ToString() == ManningGuideLineType.GrossEmployeeBenefit.ToString();

            if (isGrossEmployeeBenefitLine)
                treeNode.Override.NodeAppearance.BackColor = Constants.COLOR_ManningGuideViewLineBackground;
        }

        /// <summary>
        /// This method sets up the style of a tree node containing vacation data (vacation entitlement, vacation accrual etc.).
        /// </summary>
        /// <param name="treeNode">The tree node to set up.</param>
        /// <param name="manningGuideTabController">The controller of the manning guide tab.</param>
        private static void SetupManningGuideDataNodesForVacation(UltraTreeNode treeNode, ManningGuideTabController manningGuideTabController)
        {
            if (treeNode == null)
                throw new ArgumentNullException("treeNode");

            if (manningGuideTabController == null)
                throw new ArgumentNullException("manningGuideTabController");

            var isVacationEntitlementLine = treeNode.Cells.Count > 0 && treeNode.Cells["ManningGuideLineType"].Value.ToString() == ManningGuideLineType.VacationEntitlement.ToString();
            if (isVacationEntitlementLine)
            {
                var cell = treeNode.Cells["Name"];
                cell.Appearance.BackColor = Constants.COLOR_ManningGuideViewLineBackgroundIncludingCount;
                treeNode.Override.NodeAppearance.BackColor = Constants.COLOR_ManningGuideViewLineBackgroundIncludingCount;
            }

            var isVacationAccrualLine = treeNode.Cells.Count > 0 && treeNode.Cells["ManningGuideLineType"].Value.ToString() == ManningGuideLineType.VacationAccrual.ToString();
            if (isVacationAccrualLine)
            {
                var cell = treeNode.Cells["Name"];
                cell.Appearance.BackColor = Constants.COLOR_ManningGuideViewLineBackgroundIncludingCount;
                treeNode.Override.NodeAppearance.BackColor = Constants.COLOR_ManningGuideViewLineBackgroundIncludingCount;
            }

            var isVacationTakenNominalLine = treeNode.Cells.Count > 0 && treeNode.Cells["ManningGuideLineType"].Value.ToString() == ManningGuideLineType.VacationTakenNominal.ToString();
            if (isVacationTakenNominalLine)
            {
                var cell = treeNode.Cells["Name"];
                cell.AllowEdit = AllowCellEdit.ActivateOnly;
                cell.Appearance.BackColor = Constants.COLOR_ManningGuideViewLineBackgroundIncludingCount;
                var manningGuideId = 0;
                int.TryParse(treeNode.Cells["ManningGuideId"].Value.ToString(), out manningGuideId);                               
                var manningGuide = manningGuideTabController.AditionalData.ContainsKey(manningGuideId) ? manningGuideTabController.AditionalData[manningGuideId] : null as AdditionalManningGuideValue;                
                var allowCellEdit = manningGuide == null || manningGuide.ReplacedDuringVacation ? AllowCellEdit.ActivateOnly : AllowCellEdit.Full;

                foreach (var month in Constants.MONTH_AllForDisplaying)
                {
                    cell = treeNode.Cells[month];
                    cell.AllowEdit = allowCellEdit;

                    if (allowCellEdit == AllowCellEdit.ActivateOnly)
                        cell.Appearance.BackColor = Constants.COLOR_ManningGuideViewLineBackgroundIncludingCount;
                }
            }

            var isVacationTakenMonetaryLine = treeNode.Cells.Count > 0 && treeNode.Cells["ManningGuideLineType"].Value.ToString() == ManningGuideLineType.VacationTakenMonetary.ToString();
            if (isVacationTakenMonetaryLine)
            {
                var cell = treeNode.Cells["Name"];
                cell.Appearance.BackColor = Constants.COLOR_ManningGuideViewLineBackgroundIncludingCount;
                treeNode.Override.NodeAppearance.BackColor = Constants.COLOR_ManningGuideViewLineBackgroundIncludingCount;
            }
        }

        /// <summary>
        /// This method sets up the style of a tree node containing personal pay data (personal pay entitlement, personal pay accrual etc.).
        /// </summary>
        /// <param name="treeNode">The tree node to set up.</param>
        /// <param name="manningGuideTabController">The controller of the manning guide tab.</param>
        private static void SetupManningGuideDataNodesForPersonalPay(UltraTreeNode treeNode, ManningGuideTabController manningGuideTabController)
        {
            if (treeNode == null)
                throw new ArgumentNullException("treeNode");

            if (manningGuideTabController == null)
                throw new ArgumentNullException("manningGuideTabController");

            var isPersonalPayEntitlementLine = treeNode.Cells.Count > 0 && treeNode.Cells["ManningGuideLineType"].Value.ToString() == ManningGuideLineType.PersonalPayEntitlement.ToString();
            if (isPersonalPayEntitlementLine)
            {
                var cell = treeNode.Cells["Name"];
                cell.Appearance.BackColor = Constants.COLOR_ManningGuideViewLineBackgroundIncludingCount;
                treeNode.Override.NodeAppearance.BackColor = Constants.COLOR_ManningGuideViewLineBackgroundIncludingCount;
            }

            var isPersonalPayAccrualLine = treeNode.Cells.Count > 0 && treeNode.Cells["ManningGuideLineType"].Value.ToString() == ManningGuideLineType.PersonalPayAccrual.ToString();
            if (isPersonalPayAccrualLine)
            {
                var cell = treeNode.Cells["Name"];
                cell.Appearance.BackColor = Constants.COLOR_ManningGuideViewLineBackgroundIncludingCount;
                treeNode.Override.NodeAppearance.BackColor = Constants.COLOR_ManningGuideViewLineBackgroundIncludingCount;
            }

            var isPersonalPayTakenNominalLine = treeNode.Cells.Count > 0 && treeNode.Cells["ManningGuideLineType"].Value.ToString() == ManningGuideLineType.PersonalPayTakenNominal.ToString();
            if (isPersonalPayTakenNominalLine)
            {
                var cell = treeNode.Cells["Name"];
                cell.AllowEdit = AllowCellEdit.ActivateOnly;
                cell.Appearance.BackColor = Constants.COLOR_ManningGuideViewLineBackgroundIncludingCount;

                var manningGuideId = 0;
                int.TryParse(treeNode.Cells["ManningGuideId"].Value.ToString(), out manningGuideId);
                var manningGuide = manningGuideTabController.AditionalData.ContainsKey(manningGuideId) ? manningGuideTabController.AditionalData[manningGuideId] : null as AdditionalManningGuideValue;
                var allowCellEdit = manningGuide == null || manningGuide.ReplacedDuringVacation ? AllowCellEdit.ActivateOnly : AllowCellEdit.Full;

                foreach (var month in Constants.MONTH_AllForDisplaying)
                {
                    cell = treeNode.Cells[month];
                    cell.AllowEdit = allowCellEdit;

                    if (allowCellEdit == AllowCellEdit.ActivateOnly)
                        cell.Appearance.BackColor = Constants.COLOR_ManningGuideViewLineBackgroundIncludingCount;
                }
            }

            var isPersonalPayTakenMonetaryLine = treeNode.Cells.Count > 0 && treeNode.Cells["ManningGuideLineType"].Value.ToString() == ManningGuideLineType.PersonalPayTakenMonetary.ToString();
            if (isPersonalPayTakenMonetaryLine)
            {
                var cell = treeNode.Cells["Name"];
                cell.Appearance.BackColor = Constants.COLOR_ManningGuideViewLineBackgroundIncludingCount;
                treeNode.Override.NodeAppearance.BackColor = Constants.COLOR_ManningGuideViewLineBackgroundIncludingCount;
            }
        }

        /// <summary>
        /// This method sets up the style of a tree node containing holiday data (entitlement, accrual).
        /// </summary>
        /// <param name="treeNode">The tree node to set up.</param>
        /// <param name="manningGuideTabController">The controller of the manning guide tab.</param>
        private static void SetupManningGuideDataNodesForHolidays(UltraTreeNode treeNode, ManningGuideTabController manningGuideTabController)
        {
            if (treeNode == null)
                throw new ArgumentNullException("treeNode");

            if (manningGuideTabController == null)
                throw new ArgumentNullException("manningGuideTabController");

            var isHolidayEntitlementLine = treeNode.Cells.Count > 0 && treeNode.Cells["ManningGuideLineType"].Value.ToString() == ManningGuideLineType.HolidayEntitlement.ToString();
            if (isHolidayEntitlementLine)
            {
                var cell = treeNode.Cells["Name"];
                cell.Appearance.BackColor = Constants.COLOR_ManningGuideViewLineBackgroundIncludingCount;
                treeNode.Override.NodeAppearance.BackColor = Constants.COLOR_ManningGuideViewLineBackgroundIncludingCount;
            }

            var isHolidayAccrualLine = treeNode.Cells.Count > 0 && treeNode.Cells["ManningGuideLineType"].Value.ToString() == ManningGuideLineType.HolidayAccrual.ToString();
            if (isHolidayAccrualLine)
            {
                var cell = treeNode.Cells["Name"];
                cell.Appearance.BackColor = Constants.COLOR_ManningGuideViewLineBackgroundIncludingCount;
                treeNode.Override.NodeAppearance.BackColor = Constants.COLOR_ManningGuideViewLineBackgroundIncludingCount;
            }

            var isHolidayTakenNominalLine = treeNode.Cells.Count > 0 && treeNode.Cells["ManningGuideLineType"].Value.ToString() == ManningGuideLineType.HolidayTakenNominal.ToString();
            if (isHolidayTakenNominalLine)
            {
                var cell = treeNode.Cells["Name"];
                cell.AllowEdit = AllowCellEdit.ActivateOnly;
                cell.Appearance.BackColor = Constants.COLOR_ManningGuideViewLineBackgroundIncludingCount;

                var manningGuideId = 0;
                int.TryParse(treeNode.Cells["ManningGuideId"].Value.ToString(), out manningGuideId);
                var manningGuide = manningGuideTabController.AditionalData.ContainsKey(manningGuideId) ? manningGuideTabController.AditionalData[manningGuideId] : null as AdditionalManningGuideValue;
                var allowCellEdit = manningGuide == null || manningGuide.ReplacedDuringVacation ? AllowCellEdit.ActivateOnly : AllowCellEdit.Full;

                foreach (var month in Constants.MONTH_AllForDisplaying)
                {
                    cell = treeNode.Cells[month];
                    cell.AllowEdit = allowCellEdit;

                    if (allowCellEdit == AllowCellEdit.ActivateOnly)
                        cell.Appearance.BackColor = Constants.COLOR_ManningGuideViewLineBackgroundIncludingCount;
                }
            }

            var isHolidayTakenMonetaryLine = treeNode.Cells.Count > 0 && treeNode.Cells["ManningGuideLineType"].Value.ToString() == ManningGuideLineType.HolidayTakenMonetary.ToString();
            if (isHolidayTakenMonetaryLine)
            {
                var cell = treeNode.Cells["Name"];
                cell.Appearance.BackColor = Constants.COLOR_ManningGuideViewLineBackgroundIncludingCount;
                treeNode.Override.NodeAppearance.BackColor = Constants.COLOR_ManningGuideViewLineBackgroundIncludingCount;
            }
        }

        /// <summary>
        /// This method sets up the columns for the salary component rows.
        /// </summary>
        /// <param name="columnSet">The column set to set up.</param>
        private static void SetupSalaryComponentsColumnSet(UltraTreeColumnSet columnSet)
        {
            columnSet.Columns["Name"].AllowCellEdit = AllowCellEdit.ActivateOnly;
            columnSet.Columns["SalaryComponentId"].Visible = false;
            columnSet.Columns["ManningGuideId"].Visible = false;

            columnSet.Columns["MonthPlanningUnitId1"].Visible = false;
            columnSet.Columns["Jan"].AllowCellEdit = AllowCellEdit.Full;
            columnSet.Columns["MonthPlanningUnitId2"].Visible = false;
            columnSet.Columns["Feb"].AllowCellEdit = AllowCellEdit.Full;
            columnSet.Columns["MonthPlanningUnitId3"].Visible = false;
            columnSet.Columns["Mar"].AllowCellEdit = AllowCellEdit.Full;
            columnSet.Columns["MonthPlanningUnitId4"].Visible = false;
            columnSet.Columns["Apr"].AllowCellEdit = AllowCellEdit.Full;
            columnSet.Columns["MonthPlanningUnitId5"].Visible = false;
            columnSet.Columns["May"].AllowCellEdit = AllowCellEdit.Full;
            columnSet.Columns["MonthPlanningUnitId6"].Visible = false;
            columnSet.Columns["Jun"].AllowCellEdit = AllowCellEdit.Full;
            columnSet.Columns["MonthPlanningUnitId7"].Visible = false;
            columnSet.Columns["Jul"].AllowCellEdit = AllowCellEdit.Full;
            columnSet.Columns["MonthPlanningUnitId8"].Visible = false;
            columnSet.Columns["Aug"].AllowCellEdit = AllowCellEdit.Full;
            columnSet.Columns["MonthPlanningUnitId9"].Visible = false;
            columnSet.Columns["Sep"].AllowCellEdit = AllowCellEdit.Full;
            columnSet.Columns["MonthPlanningUnitId10"].Visible = false;
            columnSet.Columns["Oct"].AllowCellEdit = AllowCellEdit.Full;
            columnSet.Columns["MonthPlanningUnitId11"].Visible = false;
            columnSet.Columns["Nov"].AllowCellEdit = AllowCellEdit.Full;
            columnSet.Columns["MonthPlanningUnitId12"].Visible = false;
            columnSet.Columns["Dec"].AllowCellEdit = AllowCellEdit.Full;

            columnSet.Columns["Total"].AllowCellEdit = AllowCellEdit.ActivateOnly;
            columnSet.Columns["Total"].CellAppearance.BackColor = Constants.COLOR_ManningGuideTreeTotalBackground;
        }

        /// <summary>
        /// This method sets up the columns for the salary component rows.
        /// </summary>
        /// <param name="columnSet">The column set to set up.</param>
        private static void SetupEmployeeBenefitsColumnSet(UltraTreeColumnSet columnSet)
        {
            columnSet.Columns["Name"].AllowCellEdit = AllowCellEdit.ActivateOnly;
            columnSet.Columns["EmployeeBenefitId"].Visible = false;
            columnSet.Columns["SalaryComponentId"].Visible = false;

            columnSet.Columns["MonthPlanningUnitId1"].Visible = false;
            columnSet.Columns["Jan"].AllowCellEdit = AllowCellEdit.Full;
            columnSet.Columns["MonthPlanningUnitId2"].Visible = false;
            columnSet.Columns["Feb"].AllowCellEdit = AllowCellEdit.Full;
            columnSet.Columns["MonthPlanningUnitId3"].Visible = false;
            columnSet.Columns["Mar"].AllowCellEdit = AllowCellEdit.Full;
            columnSet.Columns["MonthPlanningUnitId4"].Visible = false;
            columnSet.Columns["Apr"].AllowCellEdit = AllowCellEdit.Full;
            columnSet.Columns["MonthPlanningUnitId5"].Visible = false;
            columnSet.Columns["May"].AllowCellEdit = AllowCellEdit.Full;
            columnSet.Columns["MonthPlanningUnitId6"].Visible = false;
            columnSet.Columns["Jun"].AllowCellEdit = AllowCellEdit.Full;
            columnSet.Columns["MonthPlanningUnitId7"].Visible = false;
            columnSet.Columns["Jul"].AllowCellEdit = AllowCellEdit.Full;
            columnSet.Columns["MonthPlanningUnitId8"].Visible = false;
            columnSet.Columns["Aug"].AllowCellEdit = AllowCellEdit.Full;
            columnSet.Columns["MonthPlanningUnitId9"].Visible = false;
            columnSet.Columns["Sep"].AllowCellEdit = AllowCellEdit.Full;
            columnSet.Columns["MonthPlanningUnitId10"].Visible = false;
            columnSet.Columns["Oct"].AllowCellEdit = AllowCellEdit.Full;
            columnSet.Columns["MonthPlanningUnitId11"].Visible = false;
            columnSet.Columns["Nov"].AllowCellEdit = AllowCellEdit.Full;
            columnSet.Columns["MonthPlanningUnitId12"].Visible = false;
            columnSet.Columns["Dec"].AllowCellEdit = AllowCellEdit.Full;

            columnSet.Columns["Total"].AllowCellEdit = AllowCellEdit.ActivateOnly;
            columnSet.Columns["Total"].CellAppearance.BackColor = Constants.COLOR_ManningGuideTreeTotalBackground;
        }

        /// <summary>
        /// This method sets up the columns for the manning guide rows (employees).
        /// </summary>
        /// <param name="columnSet">The column set to set up.</param>
        private static void SetupManningGuidesColumnSet(UltraTreeColumnSet columnSet)
        {
            columnSet.Columns["Name"].AllowCellEdit = AllowCellEdit.ActivateOnly;
            columnSet.Columns["ManningGuideId"].Visible = false;
            columnSet.Columns["PositionId"].Visible = false;

            columnSet.Columns["MonthPlanningUnitId1"].Visible = false;
            columnSet.Columns["Jan"].AllowCellEdit = AllowCellEdit.ActivateOnly;
            columnSet.Columns["Jan"].PerformAutoResize(ColumnAutoSizeMode.AllNodesWithDescendants);

            columnSet.Columns["MonthPlanningUnitId2"].Visible = false;
            columnSet.Columns["Feb"].AllowCellEdit = AllowCellEdit.ActivateOnly;
            columnSet.Columns["Feb"].PerformAutoResize(ColumnAutoSizeMode.AllNodesWithDescendants);

            columnSet.Columns["MonthPlanningUnitId3"].Visible = false;
            columnSet.Columns["Mar"].AllowCellEdit = AllowCellEdit.ActivateOnly;
            columnSet.Columns["Mar"].PerformAutoResize(ColumnAutoSizeMode.AllNodesWithDescendants);

            columnSet.Columns["MonthPlanningUnitId4"].Visible = false;
            columnSet.Columns["Apr"].AllowCellEdit = AllowCellEdit.ActivateOnly;
            columnSet.Columns["Apr"].PerformAutoResize(ColumnAutoSizeMode.AllNodesWithDescendants);

            columnSet.Columns["MonthPlanningUnitId5"].Visible = false;
            columnSet.Columns["May"].AllowCellEdit = AllowCellEdit.ActivateOnly;
            columnSet.Columns["May"].PerformAutoResize(ColumnAutoSizeMode.AllNodesWithDescendants);

            columnSet.Columns["MonthPlanningUnitId6"].Visible = false;
            columnSet.Columns["Jun"].AllowCellEdit = AllowCellEdit.ActivateOnly;
            columnSet.Columns["Jun"].PerformAutoResize(ColumnAutoSizeMode.AllNodesWithDescendants);

            columnSet.Columns["MonthPlanningUnitId7"].Visible = false;
            columnSet.Columns["Jul"].AllowCellEdit = AllowCellEdit.ActivateOnly;
            columnSet.Columns["Jul"].PerformAutoResize(ColumnAutoSizeMode.AllNodesWithDescendants);

            columnSet.Columns["MonthPlanningUnitId8"].Visible = false;
            columnSet.Columns["Aug"].AllowCellEdit = AllowCellEdit.ActivateOnly;
            columnSet.Columns["Aug"].PerformAutoResize(ColumnAutoSizeMode.AllNodesWithDescendants);

            columnSet.Columns["MonthPlanningUnitId9"].Visible = false;
            columnSet.Columns["Sep"].AllowCellEdit = AllowCellEdit.ActivateOnly;
            columnSet.Columns["Sep"].PerformAutoResize(ColumnAutoSizeMode.AllNodesWithDescendants);

            columnSet.Columns["MonthPlanningUnitId10"].Visible = false;
            columnSet.Columns["Oct"].AllowCellEdit = AllowCellEdit.ActivateOnly;
            columnSet.Columns["Oct"].PerformAutoResize(ColumnAutoSizeMode.AllNodesWithDescendants);

            columnSet.Columns["MonthPlanningUnitId11"].Visible = false;
            columnSet.Columns["Nov"].AllowCellEdit = AllowCellEdit.ActivateOnly;
            columnSet.Columns["Nov"].PerformAutoResize(ColumnAutoSizeMode.AllNodesWithDescendants);

            columnSet.Columns["MonthPlanningUnitId12"].Visible = false;
            columnSet.Columns["Dec"].AllowCellEdit = AllowCellEdit.ActivateOnly;
            columnSet.Columns["Dec"].PerformAutoResize(ColumnAutoSizeMode.AllNodesWithDescendants);
            
            columnSet.Columns["Total"].AllowCellEdit = AllowCellEdit.ActivateOnly;
            columnSet.Columns["Total"].CellAppearance.BackColor = Constants.COLOR_ManningGuideTreeTotalBackground;
        }

        /// <summary>
        /// This method sets up the columns for the position rows.
        /// </summary>
        /// <param name="columnSet">The column set to set up.</param>
        private static void SetupPositionsColumnSet(UltraTreeColumnSet columnSet)
        {
            columnSet.NodeTextColumn = columnSet.Columns["Name"];

            columnSet.Columns["Name"].AllowCellEdit = AllowCellEdit.ActivateOnly;
            columnSet.Columns["PositionId"].Visible = false;
            columnSet.Columns["DepartmentId"].Visible = false;

            columnSet.Columns["MonthPlanningUnitId1"].Visible = false;
            columnSet.Columns["Jan"].AllowCellEdit = AllowCellEdit.ActivateOnly;
            columnSet.Columns["Jan"].DataType = typeof(decimal);
            columnSet.Columns["Jan"].CellAppearance.TextHAlign = Infragistics.Win.HAlign.Right;

            columnSet.Columns["MonthPlanningUnitId2"].Visible = false;
            columnSet.Columns["Feb"].AllowCellEdit = AllowCellEdit.ActivateOnly;
            columnSet.Columns["Feb"].DataType = typeof(decimal);
            columnSet.Columns["Feb"].CellAppearance.TextHAlign = Infragistics.Win.HAlign.Right;

            columnSet.Columns["MonthPlanningUnitId3"].Visible = false;
            columnSet.Columns["Mar"].AllowCellEdit = AllowCellEdit.ActivateOnly;
            columnSet.Columns["Mar"].DataType = typeof(decimal);
            columnSet.Columns["Mar"].CellAppearance.TextHAlign = Infragistics.Win.HAlign.Right;

            columnSet.Columns["MonthPlanningUnitId4"].Visible = false;
            columnSet.Columns["Apr"].AllowCellEdit = AllowCellEdit.ActivateOnly;
            columnSet.Columns["Apr"].DataType = typeof(decimal);
            columnSet.Columns["Apr"].CellAppearance.TextHAlign = Infragistics.Win.HAlign.Right;

            columnSet.Columns["MonthPlanningUnitId5"].Visible = false;
            columnSet.Columns["May"].AllowCellEdit = AllowCellEdit.ActivateOnly;
            columnSet.Columns["May"].DataType = typeof(decimal);
            columnSet.Columns["May"].CellAppearance.TextHAlign = Infragistics.Win.HAlign.Right;

            columnSet.Columns["MonthPlanningUnitId6"].Visible = false;
            columnSet.Columns["Jun"].AllowCellEdit = AllowCellEdit.ActivateOnly;
            columnSet.Columns["Jun"].DataType = typeof(decimal);
            columnSet.Columns["Jun"].CellAppearance.TextHAlign = Infragistics.Win.HAlign.Right;

            columnSet.Columns["MonthPlanningUnitId7"].Visible = false;
            columnSet.Columns["Jul"].AllowCellEdit = AllowCellEdit.ActivateOnly;
            columnSet.Columns["Jul"].DataType = typeof(decimal);
            columnSet.Columns["Jul"].CellAppearance.TextHAlign = Infragistics.Win.HAlign.Right;

            columnSet.Columns["MonthPlanningUnitId8"].Visible = false;
            columnSet.Columns["Aug"].AllowCellEdit = AllowCellEdit.ActivateOnly;
            columnSet.Columns["Aug"].DataType = typeof(decimal);
            columnSet.Columns["Aug"].CellAppearance.TextHAlign = Infragistics.Win.HAlign.Right;

            columnSet.Columns["MonthPlanningUnitId9"].Visible = false;
            columnSet.Columns["Sep"].AllowCellEdit = AllowCellEdit.ActivateOnly;
            columnSet.Columns["Sep"].DataType = typeof(decimal);
            columnSet.Columns["Sep"].CellAppearance.TextHAlign = Infragistics.Win.HAlign.Right;

            columnSet.Columns["MonthPlanningUnitId10"].Visible = false;
            columnSet.Columns["Oct"].AllowCellEdit = AllowCellEdit.ActivateOnly;
            columnSet.Columns["Oct"].DataType = typeof(decimal);
            columnSet.Columns["Oct"].CellAppearance.TextHAlign = Infragistics.Win.HAlign.Right;

            columnSet.Columns["MonthPlanningUnitId11"].Visible = false;
            columnSet.Columns["Nov"].AllowCellEdit = AllowCellEdit.ActivateOnly;
            columnSet.Columns["Nov"].DataType = typeof(decimal);
            columnSet.Columns["Nov"].CellAppearance.TextHAlign = Infragistics.Win.HAlign.Right;

            columnSet.Columns["MonthPlanningUnitId12"].Visible = false;
            columnSet.Columns["Dec"].AllowCellEdit = AllowCellEdit.ActivateOnly;
            columnSet.Columns["Dec"].DataType = typeof(decimal);
            columnSet.Columns["Dec"].CellAppearance.TextHAlign = Infragistics.Win.HAlign.Right;

            columnSet.Columns["Total"].AllowCellEdit = AllowCellEdit.ActivateOnly;
            columnSet.Columns["Total"].DataType = typeof(decimal);
            columnSet.Columns["Total"].CellAppearance.TextHAlign = Infragistics.Win.HAlign.Right;
            columnSet.Columns["Total"].CellAppearance.BackColor = Constants.COLOR_ManningGuideTreeTotalBackground;
        }

        /// <summary>
        /// This method sets up the columns for the input tools.
        /// </summary>
        /// <param name="columnSet">The column set to set up.</param>
        private static void SetupInputToolsColumnSet(UltraTreeColumnSet columnSet)
        {
            columnSet.NodeTextColumn = columnSet.Columns["Name"];

            columnSet.Columns["Name"].AllowCellEdit = AllowCellEdit.ActivateOnly;
            columnSet.Columns["InputToolId"].Visible = false;
            columnSet.Columns["DepartmentId"].Visible = false;

            columnSet.Columns["MonthPlanningUnitId1"].Visible = false;
            columnSet.Columns["Jan"].AllowCellEdit = AllowCellEdit.Full;
            columnSet.Columns["Jan"].DataType = typeof(decimal);
            columnSet.Columns["Jan"].CellAppearance.TextHAlign = Infragistics.Win.HAlign.Right;

            columnSet.Columns["MonthPlanningUnitId2"].Visible = false;
            columnSet.Columns["Feb"].AllowCellEdit = AllowCellEdit.Full;
            columnSet.Columns["Feb"].DataType = typeof(decimal);
            columnSet.Columns["Feb"].CellAppearance.TextHAlign = Infragistics.Win.HAlign.Right;

            columnSet.Columns["MonthPlanningUnitId3"].Visible = false;
            columnSet.Columns["Mar"].AllowCellEdit = AllowCellEdit.Full;
            columnSet.Columns["Mar"].DataType = typeof(decimal);
            columnSet.Columns["Mar"].CellAppearance.TextHAlign = Infragistics.Win.HAlign.Right;

            columnSet.Columns["MonthPlanningUnitId4"].Visible = false;
            columnSet.Columns["Apr"].AllowCellEdit = AllowCellEdit.Full;
            columnSet.Columns["Apr"].DataType = typeof(decimal);
            columnSet.Columns["Apr"].CellAppearance.TextHAlign = Infragistics.Win.HAlign.Right;

            columnSet.Columns["MonthPlanningUnitId5"].Visible = false;
            columnSet.Columns["May"].AllowCellEdit = AllowCellEdit.Full;
            columnSet.Columns["May"].DataType = typeof(decimal);
            columnSet.Columns["May"].CellAppearance.TextHAlign = Infragistics.Win.HAlign.Right;

            columnSet.Columns["MonthPlanningUnitId6"].Visible = false;
            columnSet.Columns["Jun"].AllowCellEdit = AllowCellEdit.Full;
            columnSet.Columns["Jun"].DataType = typeof(decimal);
            columnSet.Columns["Jun"].CellAppearance.TextHAlign = Infragistics.Win.HAlign.Right;

            columnSet.Columns["MonthPlanningUnitId7"].Visible = false;
            columnSet.Columns["Jul"].AllowCellEdit = AllowCellEdit.Full;
            columnSet.Columns["Jul"].DataType = typeof(decimal);
            columnSet.Columns["Jul"].CellAppearance.TextHAlign = Infragistics.Win.HAlign.Right;

            columnSet.Columns["MonthPlanningUnitId8"].Visible = false;
            columnSet.Columns["Aug"].AllowCellEdit = AllowCellEdit.Full;
            columnSet.Columns["Aug"].DataType = typeof(decimal);
            columnSet.Columns["Aug"].CellAppearance.TextHAlign = Infragistics.Win.HAlign.Right;

            columnSet.Columns["MonthPlanningUnitId9"].Visible = false;
            columnSet.Columns["Sep"].AllowCellEdit = AllowCellEdit.Full;
            columnSet.Columns["Sep"].DataType = typeof(decimal);
            columnSet.Columns["Sep"].CellAppearance.TextHAlign = Infragistics.Win.HAlign.Right;

            columnSet.Columns["MonthPlanningUnitId10"].Visible = false;
            columnSet.Columns["Oct"].AllowCellEdit = AllowCellEdit.Full;
            columnSet.Columns["Oct"].DataType = typeof(decimal);
            columnSet.Columns["Oct"].CellAppearance.TextHAlign = Infragistics.Win.HAlign.Right;

            columnSet.Columns["MonthPlanningUnitId11"].Visible = false;
            columnSet.Columns["Nov"].AllowCellEdit = AllowCellEdit.Full;
            columnSet.Columns["Nov"].DataType = typeof(decimal);
            columnSet.Columns["Nov"].CellAppearance.TextHAlign = Infragistics.Win.HAlign.Right;

            columnSet.Columns["MonthPlanningUnitId12"].Visible = false;
            columnSet.Columns["Dec"].AllowCellEdit = AllowCellEdit.Full;
            columnSet.Columns["Dec"].DataType = typeof(decimal);
            columnSet.Columns["Dec"].CellAppearance.TextHAlign = Infragistics.Win.HAlign.Right;

            columnSet.Columns["Total"].AllowCellEdit = AllowCellEdit.ActivateOnly;
            columnSet.Columns["Total"].DataType = typeof(decimal);
            columnSet.Columns["Total"].CellAppearance.TextHAlign = Infragistics.Win.HAlign.Right;
            columnSet.Columns["Total"].CellAppearance.BackColor = Constants.COLOR_ManningGuideTreeTotalBackground;
        }

        /// <summary>
        /// This method sets up the columns for the department row.
        /// </summary>
        /// <param name="columnSet">The column set to set up.</param>
        /// <param name="application">The current application.</param>
        /// <param name="selectedYear">The current selected year in the manning guide.</param>
        private static void SetupDepartmentsColumnSet(UltraTreeColumnSet columnSet, IApplication application, int selectedYear)
        {
            if (application==null)
                throw new ArgumentNullException("application");

            var fiscalMonthCaptions = FiscalUtils.GetFiscalMonthCaptions(application, selectedYear).ToArray();

            columnSet.NodeTextColumn = columnSet.Columns["Name"];

            columnSet.Columns["Name"].AllowCellEdit = AllowCellEdit.ActivateOnly;
            columnSet.Columns["Id"].Visible = false;
            columnSet.Columns["DepartmentId"].Visible = false;

            columnSet.Columns["MonthPlanningUnitId1"].Visible = false;
            columnSet.Columns["Jan"].Text = fiscalMonthCaptions[0];
            columnSet.Columns["Jan"].AllowCellEdit = AllowCellEdit.ActivateOnly;
            columnSet.Columns["Jan"].CellAppearance.TextHAlign = Infragistics.Win.HAlign.Right;

            columnSet.Columns["MonthPlanningUnitId2"].Visible = false;
            columnSet.Columns["Feb"].Text = fiscalMonthCaptions[1];
            columnSet.Columns["Feb"].AllowCellEdit = AllowCellEdit.ActivateOnly;
            columnSet.Columns["Feb"].CellAppearance.TextHAlign = Infragistics.Win.HAlign.Right;

            columnSet.Columns["MonthPlanningUnitId3"].Visible = false;
            columnSet.Columns["Mar"].Text = fiscalMonthCaptions[2];
            columnSet.Columns["Mar"].AllowCellEdit = AllowCellEdit.ActivateOnly;
            columnSet.Columns["Mar"].CellAppearance.TextHAlign = Infragistics.Win.HAlign.Right;

            columnSet.Columns["MonthPlanningUnitId4"].Visible = false;
            columnSet.Columns["Apr"].Text = fiscalMonthCaptions[3];
            columnSet.Columns["Apr"].AllowCellEdit = AllowCellEdit.ActivateOnly;
            columnSet.Columns["Apr"].CellAppearance.TextHAlign = Infragistics.Win.HAlign.Right;

            columnSet.Columns["MonthPlanningUnitId5"].Visible = false;
            columnSet.Columns["May"].Text = fiscalMonthCaptions[4];
            columnSet.Columns["May"].AllowCellEdit = AllowCellEdit.ActivateOnly;
            columnSet.Columns["May"].CellAppearance.TextHAlign = Infragistics.Win.HAlign.Right;

            columnSet.Columns["MonthPlanningUnitId6"].Visible = false;
            columnSet.Columns["Jun"].Text = fiscalMonthCaptions[5];
            columnSet.Columns["Jun"].AllowCellEdit = AllowCellEdit.ActivateOnly;
            columnSet.Columns["Jun"].CellAppearance.TextHAlign = Infragistics.Win.HAlign.Right;

            columnSet.Columns["MonthPlanningUnitId7"].Visible = false;
            columnSet.Columns["Jul"].Text = fiscalMonthCaptions[6];
            columnSet.Columns["Jul"].AllowCellEdit = AllowCellEdit.ActivateOnly;
            columnSet.Columns["Jul"].CellAppearance.TextHAlign = Infragistics.Win.HAlign.Right;

            columnSet.Columns["MonthPlanningUnitId8"].Visible = false;
            columnSet.Columns["Aug"].Text = fiscalMonthCaptions[7];
            columnSet.Columns["Aug"].AllowCellEdit = AllowCellEdit.ActivateOnly;
            columnSet.Columns["Aug"].CellAppearance.TextHAlign = Infragistics.Win.HAlign.Right;

            columnSet.Columns["MonthPlanningUnitId9"].Visible = false;
            columnSet.Columns["Sep"].Text = fiscalMonthCaptions[8];
            columnSet.Columns["Sep"].AllowCellEdit = AllowCellEdit.ActivateOnly;
            columnSet.Columns["Sep"].CellAppearance.TextHAlign = Infragistics.Win.HAlign.Right;

            columnSet.Columns["MonthPlanningUnitId10"].Visible = false;
            columnSet.Columns["Oct"].Text = fiscalMonthCaptions[9];
            columnSet.Columns["Oct"].AllowCellEdit = AllowCellEdit.ActivateOnly;
            columnSet.Columns["Oct"].CellAppearance.TextHAlign = Infragistics.Win.HAlign.Right;

            columnSet.Columns["MonthPlanningUnitId11"].Visible = false;
            columnSet.Columns["Nov"].Text = fiscalMonthCaptions[10];
            columnSet.Columns["Nov"].AllowCellEdit = AllowCellEdit.ActivateOnly;
            columnSet.Columns["Nov"].CellAppearance.TextHAlign = Infragistics.Win.HAlign.Right;

            columnSet.Columns["MonthPlanningUnitId12"].Visible = false;
            columnSet.Columns["Dec"].Text = fiscalMonthCaptions[11];
            columnSet.Columns["Dec"].AllowCellEdit = AllowCellEdit.ActivateOnly;
            columnSet.Columns["Dec"].CellAppearance.TextHAlign = Infragistics.Win.HAlign.Right;

            columnSet.Columns["Total"].AllowCellEdit = AllowCellEdit.ActivateOnly;
            columnSet.Columns["Total"].DataType = typeof(decimal);
            columnSet.Columns["Total"].CellAppearance.TextHAlign = Infragistics.Win.HAlign.Right;
            columnSet.Columns["Total"].CellAppearance.BackColor = Constants.COLOR_ManningGuideTreeTotalBackground;
        }

        /// <summary>
        /// This method checks if the employee is valid in the given month, and then activates or deactivates the column appropriate.
        /// </summary>
        /// <param name="treeNode">The node containing the month cell.</param>
        /// <param name="idColumn">The column containing the id of the month planning unit (invisible).</param>
        /// <param name="monthColumn">The column containing the month planning unit value.</param>
        /// <param name="manningGuideTabController">The controller of the manning guide.</param>
        private static void SetMonthAvailability(UltraTreeNode treeNode, string idColumn, string monthColumn, ManningGuideTabController manningGuideTabController)
        {
            if (treeNode == null)
                throw new ArgumentNullException("treeNode");

            if (manningGuideTabController == null)
                throw new ArgumentNullException("manningGuideTabController");

            var id = 0;
            if (!int.TryParse(treeNode.Cells[idColumn].Text, out id))
                return;

            var unit = manningGuideTabController.AditionalPlanningData.ContainsKey(id) ? manningGuideTabController.AditionalPlanningData[id] : null;

            if (unit == null)
                return;

            var active = unit.IsActive && !manningGuideTabController.ManningGuideClosed;

            var isReadOnlyCell = unit.SalaryTypeId == Constants.ID_SalaryComponentTypeCostPerHour;

            if (active)
            {
                treeNode.Cells[monthColumn].Appearance.ForeColor = Color.Black;

                if (isReadOnlyCell)
                {
                    treeNode.Cells[idColumn].AllowEdit = AllowCellEdit.ReadOnly;
                    treeNode.Cells[monthColumn].AllowEdit = AllowCellEdit.ReadOnly;
                }
                else
                {
                    treeNode.Cells[idColumn].AllowEdit = AllowCellEdit.Full;
                    treeNode.Cells[monthColumn].AllowEdit = AllowCellEdit.Full;
                }
            }
            else
            {
                treeNode.Cells[monthColumn].Appearance.ForeColor = Color.Gray;
                treeNode.Cells[idColumn].AllowEdit = AllowCellEdit.ActivateOnly;
                treeNode.Cells[monthColumn].AllowEdit = AllowCellEdit.ActivateOnly;
            }
        }

        #endregion
    }
}
